/*
    This file is part of the UNF library / toolset
    Copyright (C) 2000 Steffen Kaiser

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*	NLSUPTST	- NLS UpCase function tester
 *	Tests the DOS API functions DOS-65-2[0-2]
 *	Useage:
 *	1)	NLSUPTST /c
 *	2)	NLSUPTST
 *
 *	Variant 1) creates a test file %countryCode%-%codepageID%.UP;
 *	variant 2) runs the test upon that file.
 */

#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <stdarg.h>

#include "../unf.h"

#define FP_OFF(p)		(unsigned short)(p)
#define FP_SEG(p)		(unsigned short)((unsigned long)(void far *)(p) >> 16)
	/* Has been renamed to avoid name clashes */
#define	error unf__eerror

#ifndef lint
static char const rcsid[] = 
	"$Id: nlsuptst.c 1.1 2000/07/02 04:51:06 ska Exp ska $";
#endif

char buf[258];
char zbuf[258];

char fnam[] = "cntry-cp.UP";

struct nlsExtCntryInfo eci;
struct UNF_IN_BUFFER iBuf;
#define get(a) unf_get(a, &iBuf)

main(int argc, char **argv)
{	
	unsigned i;
	FILE *f;
	int err;
#if !defined(__TURBOC__)
    union REGS r;
    struct SREGS sr;
#endif

#define cntry countryCode
#define cp codePage

		/* retreive extended country information */
	if(get(1)) {
		fputs("Cannot proceed, sorry!\n", stderr);
		return 20;
	}
		/* The extended country inormation contains 10 "dummy" bytes
			at its end */
	if(iBuf.info.subInfo.size > sizeof(eci) - 3 + 10) {
		fprintf(stderr, "ExtCntryInfo buffer overflow (%u)\n", iBuf.info.subInfo.size);
		return 30;
	}
	memcpy(&eci, &iBuf, sizeof(eci));

	printf("Country ID: %u, Codepage ID: %u\n", eci.cntry, eci.cp);
	if(eci.cntry > 999 && eci.cp > 999) {
		fputs("Both IDs exceeds 999; cannot proceed\n", stderr);
		return 31;
	}
	sprintf(fnam, "%03u-%03u.UP", eci.cntry, eci.cp);

	if(argc == 2 && strcmpi(argv[1], "/c") == 0) {
		if((f = fopen(fnam, "wb")) == NULL) {
			fprintf(stderr, "Cannot create file: %s\n", fnam);
			return 40;
		}
		for(i = 0; i <= 255; ++i) {
#if defined(__TURBOC__)
			_DL = i;
			_AX = 0x6520;
			geninterrupt(0x21);
			buf[i] = _DL;
#else
            r.h.dl = i;
            r.x.ax = 0x6520;
            intdos(&r, &r);
            buf[i] = r.h.dl;
#endif
		}
		if(fwrite(buf, sizeof(buf), 1, f) != 1) {
			fprintf(stderr, "Cannot write to file: %s\n", fnam);
			fclose(f);
			return 41;
		}
		fclose(f);
		printf("Upcase NLS test file \"%s\" successfully created\n"
		 , fnam);
	} else if(argc > 1) {
		puts("NLSUPTST - NLS UpCase function tester\n"
			"Useage:	NLSUPTST /c\n"
			"\t-or-\n\t\tNLSUPTST\n"
			"First variant creates a NLSUPTST file, second variant\n"
			"probes this file against the current NLS.\n"
			"The test files are automatically named:\n"
			"\t<country_code>-<codepage_ID>.UP");
		return 127;
	} else {		/* Probe */
		if((f = fopen(fnam, "rb")) == NULL) {
			fprintf(stderr, "Cannot open NLSUPTST file: %s\n", fnam);
			return 50;
		}
		if(fread(zbuf, sizeof(zbuf), 1, f) != 1) {
			fprintf(stderr, "Cannot read from file: %s\n"
				"Or specified file is no NLSUPTST file\n", fnam);
			fclose(f);
			return 51;
		}
		fclose(f);
		err = 0;
#if defined(__TURBOC__)
		for(i = 0; i <= 255; ++i) {
			_DL = i;
			_AX = 0x6520;
			geninterrupt(0x21);
			buf[i] = _DL;
		}
		if(memcmp(buf, zbuf, sizeof(buf)) != 0) {
			fputs("DOS-65-20 (upcase single character) failed\n", stderr);
			++err;
		}
		for(i = 0; i <= 255; ++i)
			buf[i] = i;
		_DX = FP_OFF(buf);
		_CX = 256;
		_AX = 0x6521;
		geninterrupt(0x21);
		if(memcmp(buf, zbuf, sizeof(buf)) != 0) {
			fputs("DOS-65-21 (upcase memory area) failed\n", stderr);
			++err;
		}
		for(i = 0; i <= 255; ++i)
			buf[i] = i;
		buf[256] = 0;
		_DX = FP_OFF(buf) + 1;
		_AX = 0x6522;
		geninterrupt(0x21);
#else
        for(i = 0; i <= 255; ++i) {
            r.h.dl = i;
            r.x.ax = 0x6520;
            intdos(&r, &r);
            buf[i] = r.h.dl;
        }
        if(memcmp(buf, zbuf, sizeof(buf)) != 0) {
            fputs("DOS-65-20 (upcase single character) failed\n", stderr);
            ++err;
        }
        for(i = 0; i <= 255; ++i)
            buf[i] = i;
        r.x.dx = FP_OFF(buf);
        sr.ds = sr.es = FP_SEG((char far *)buf);
        r.x.cx = 256;
        r.x.ax = 0x6521;
        intdosx(&r, &r, &sr);
        if(memcmp(buf, zbuf, sizeof(buf)) != 0) {
            fputs("DOS-65-21 (upcase memory area) failed\n", stderr);
            ++err;
        }
        for(i = 0; i <= 255; ++i)
            buf[i] = i;
        buf[256] = 0;
        r.x.dx = FP_OFF(buf) + 1;
        sr.ds = sr.es = FP_SEG((char far *)buf);
        r.x.ax = 0x6522;
        intdosx(&r, &r, &sr);
#endif
		if(memcmp(buf, zbuf, sizeof(buf)) != 0) {
			fputs("DOS-65-22 (upcase string) failed\n", stderr);
			++err;
		}
		if(!err)
			puts("NLS passed all DOS-65-2[0-2] tests\n"
				"Note: In order to pass all tests run GRAB_UNF to\n"
				"generate an .UNF file from the current NLS package\n"
				"and compare its output with a good one.");
	}
	return 0;
}
